require_relative 'player'
require_relative 'clumsy_player'
require_relative 'berserk_player'
require_relative 'die'
require_relative 'game_turn'
require_relative 'treasure_trove'
require 'csv'

module StudioGame
	
	class Game

		def initialize(title)
			@title = title.capitalize
			@players = []
		end

		def add_player(player)
			@players << player
		end

		def print_player_stats(players, type)
			puts "\n#{players.size} #{type} players:"
			players.each { |p| puts "#{p.name} (#{p.health})"}
		end


		def high_score_entry(player)
			"#{player.name.ljust(20, '.')} #{player.score}"	
		end

		def high_scores
			"\n#{@title} High Scores:"
			@players.sort.each do |p| 
				"#{p.name.ljust(20, '.')} #{p.score}"
			end
		end

		def load_players(from_file)
			CSV.foreach(from_file) do |line|
				player = Player.new(line[0], line[1].to_i)
				add_player(player)
			end
		end

		def save_high_scores(to_file="high_scores.txt")
			File.open(to_file, 'w+') do |file|
				file.puts "#{@title} High Scores:"
				@players.sort.each do |player|
					file.puts high_score_entry(player)
				end
			end
		end

		def print_stats
			strong_players, wimpy_players = @players.partition { |pl| pl.strong? }

			puts "\n#{@title} Statistics:"
			print_player_stats(strong_players, 'strong')
			print_player_stats(wimpy_players, 'wimpy')

			puts "\n#{total_points} total points from treasures found"
			puts "\n#{@title} High Scores"
			@players.sort.each do |p| 
				puts high_score_entry(p)
			end

			@players.each do |player|
				puts "\n#{player.name}'s points totals:"
				player.each_found_treasure do |treasure|
					puts "#{treasure.points} total #{treasure.name} points"
				end
				puts "#{player.points} grand total points"
			end

		end

		def total_points
			@players.reduce(0) { |sum, player| sum + player.points }
		end

		def play(rounds)
			puts "There are #{@players.count} players in #{@title}"
			@players.each do |p|
				puts p
			end

			treasures = TreasureTrove::TREASURES

			puts "\nThere are #{treasures.size} to be found:"
			treasures.each do |treasure|
				puts "A #{treasure.name} is worth #{treasure.points} points"
			end

			1.upto(rounds) do |n|
				if block_given?
					break puts "\nSTOPPING! Max score limit reached" if yield
				end
				puts "\nRound #{n}"
				@players.each do |p|
					GameTurn.take_turn(p)
				
					puts p
				end
			end
		end
	end

	if __FILE__ == $0
		# put example code here
	end
end